@@ -1,6 +1,6 @@
 /*
  *  Phusion Passenger - https://www.phusionpassenger.com/
- *  Copyright (c) 2011-2017 Phusion Holding B.V.
+ *  Copyright (c) 2011-2018 Phusion Holding B.V.
  *
  *  "Passenger", "Phusion Passenger" and "Union Station" are registered
  *  trademarks of Phusion Holding B.V.
@@ -24,7 +24,7 @@
  *  THE SOFTWARE.
  */
 #include <Core/Controller.h>
-#include <Utils/SystemTime.h>
+#include <SystemTools/SystemTime.h>
 
 /*************************************************************************
  *
@@ -167,7 +167,7 @@ Controller::sendHeaderToAppWithSessionProtocol(Client *client, Request *req) {
 		deltaMonotonic = boost::to_string(-diff);
 	}
 
-	unsigned int bufferSize = determineHeaderSizeForSessionProtocol(req,
+	unsigned int bufferSize = determineMaxHeaderSizeForSessionProtocol(req,
 		state, deltaMonotonic);
 	MemoryKit::mbuf_pool &mbuf_pool = getContext()->mbuf_pool;
 	const unsigned int MBUF_MAX_SIZE = mbuf_pool_data_size(&mbuf_pool);
@@ -317,7 +317,7 @@ httpHeaderToScgiUpperCase(unsigned char *data, unsigned int size) {
 }
 
 unsigned int
-Controller::determineHeaderSizeForSessionProtocol(Request *req,
+Controller::determineMaxHeaderSizeForSessionProtocol(Request *req,
 	SessionProtocolWorkingState &state, string delta_monotonic)
 {
 	unsigned int dataSize = sizeof(boost::uint32_t);
@@ -563,15 +563,15 @@ Controller::constructHeaderForSessionProtocol(Request *req, char * restrict buff
 
 	ServerKit::HeaderTable::Iterator it(req->headers);
 	while (*it != NULL) {
-		// This header-skipping is not accounted for in determineHeaderSizeForSessionProtocol(), but
+		// This header-skipping is not accounted for in determineMaxHeaderSizeForSessionProtocol(), but
 		// since we are only reducing the size it just wastes some mem bytes.
 		if ((
 				(it->header->hash == HTTP_CONTENT_LENGTH.hash()
 						|| it->header->hash == HTTP_CONTENT_TYPE.hash()
 						|| it->header->hash == HTTP_CONNECTION.hash()
-				) && (psg_lstr_cmp(&it->header->key, P_STATIC_STRING("content-type"))
-						|| psg_lstr_cmp(&it->header->key, P_STATIC_STRING("content-length"))
-						|| psg_lstr_cmp(&it->header->key, P_STATIC_STRING("connection"))
+				) && (psg_lstr_cmp(&it->header->key, HTTP_CONTENT_TYPE)
+						|| psg_lstr_cmp(&it->header->key, HTTP_CONTENT_LENGTH)
+						|| psg_lstr_cmp(&it->header->key, HTTP_CONNECTION)
 				)
 			) || containsNonAlphaNumDash(it->header->key)
 		   )
@@ -756,7 +756,7 @@ Controller::constructHeaderBuffersForHttpProtocol(Request *req, struct iovec *bu
 	while (*it != NULL) {
 		if ((it->header->hash == HTTP_CONNECTION.hash()
 		  || it->header->hash == ServerKit::HTTP_SET_COOKIE.hash())
-		 && (psg_lstr_cmp(&it->header->key, P_STATIC_STRING("connection"))
+		 && (psg_lstr_cmp(&it->header->key, HTTP_CONNECTION)
 		  || psg_lstr_cmp(&it->header->key, ServerKit::HTTP_SET_COOKIE)))
 		{
 			it.next();
