@@ -81,8 +81,8 @@
 #include <ConfigKit/SubComponentUtils.h>
 #include <ServerKit/Server.h>
 #include <ServerKit/AcceptLoadBalancer.h>
-#include <ConfigKit/VariantMapUtils.h>
-#include <MessageReadersWriters.h>
+#include <AppTypeDetector/Detector.h>
+#include <IOTools/MessageSerialization.h>
 #include <FileDescriptor.h>
 #include <ResourceLocator.h>
 #include <BackgroundEventLoop.cpp>
@@ -91,8 +91,7 @@
 #include <Exceptions.h>
 #include <Utils.h>
 #include <Utils/Timer.h>
-#include <Utils/MessageIO.h>
-#include <Utils/VariantMap.h>
+#include <IOTools/MessageIO.h>
 #include <Core/OptionParser.h>
 #include <Core/Controller.h>
 #include <Core/ApiServer.h>
@@ -100,6 +99,7 @@
 #include <Core/ConfigChange.h>
 #include <Core/ApplicationPool/Pool.h>
 #include <Core/SecurityUpdateChecker.h>
+#include <Core/TelemetryCollector.h>
 #include <Core/AdminPanelConnector.h>
 
 using namespace boost;
@@ -167,6 +167,7 @@ namespace Core {
 		oxt::thread *prestarterThread;
 
 		SecurityUpdateChecker *securityUpdateChecker;
+		TelemetryCollector *telemetryCollector;
 		AdminPanelConnector *adminPanelConnector;
 		oxt::thread *adminPanelConnectorThread;
 
@@ -177,6 +178,7 @@ namespace Core {
 			  shutdownCounter(0),
 			  prestarterThread(NULL),
 			  securityUpdateChecker(NULL),
+			  telemetryCollector(NULL),
 			  adminPanelConnector(NULL),
 			  adminPanelConnectorThread(NULL)
 			  /*******************/
@@ -192,6 +194,7 @@ namespace Core {
 			delete adminPanelConnectorThread;
 			delete adminPanelConnector;
 			delete securityUpdateChecker;
+			delete telemetryCollector;
 
 			/*******************/
 			/*******************/
@@ -213,6 +216,7 @@ namespace Core {
 
 using namespace Passenger::Core;
 
+static WrapperRegistry::Registry *coreWrapperRegistry;
 static Schema *coreSchema;
 static ConfigKit::Store *coreConfig;
 static WorkingObjects *workingObjects;
@@ -259,9 +263,9 @@ initializeSingleAppMode() {
 
 	if (coreConfig->get("single_app_mode_app_type").isNull()) {
 		P_DEBUG("Autodetecting application type...");
-		AppTypeDetector detector(NULL, 0);
-		PassengerAppType appTypeEnum = detector.checkAppRoot(appRoot);
-		if (appTypeEnum == PAT_NONE || appTypeEnum == PAT_ERROR) {
+		AppTypeDetector::Detector detector(*coreWrapperRegistry, NULL, 0);
+		AppTypeDetector::Detector::Result result = detector.checkAppRoot(appRoot);
+		if (result.isNull()) {
 			fprintf(stderr, "ERROR: unable to autodetect what kind of application "
 				"lives in %s. Please specify information about the app using "
 				"--app-type and --startup-file, or specify a correct location to "
@@ -271,13 +275,18 @@ initializeSingleAppMode() {
 			exit(1);
 		}
 
-		appType = getAppTypeName(appTypeEnum);
+		appType = result.wrapperRegistryEntry->language;
 	} else {
 		appType = coreConfig->get("single_app_mode_app_type").asString();
 	}
 
 	if (coreConfig->get("single_app_mode_startup_file").isNull()) {
-		startupFile = appRoot + "/" + getAppTypeStartupFile(getAppType(appType));
+		const WrapperRegistry::Entry &entry = coreWrapperRegistry->lookup(appType);
+		if (entry.defaultStartupFiles.empty()) {
+			startupFile = appRoot + "/";
+		} else {
+			startupFile = appRoot + "/" + entry.defaultStartupFiles[0];
+		}
 	} else {
 		startupFile = coreConfig->get("single_app_mode_startup_file").asString();
 	}
@@ -553,39 +562,62 @@ printInfoInThread() {
 }
 
 static void
-dumpDiagnosticsOnCrash(void *userData) {
+dumpOxtBacktracesOnCrash(void *userData) {
+	cerr << oxt::thread::all_backtraces();
+	cerr.flush();
+}
+
+static void
+dumpControllerStatesOnCrash(void *userData) {
 	WorkingObjects *wo = workingObjects;
 	unsigned int i;
 
-	cerr << "### Backtraces\n";
-	cerr << oxt::thread::all_backtraces();
-	cerr.flush();
-
 	for (i = 0; i < wo->threadWorkingObjects.size(); i++) {
 		ThreadWorkingObjects *two = &wo->threadWorkingObjects[i];
-		cerr << "### Request handler state (thread " << (i + 1) << ")\n";
+		cerr << "####### Controller state (thread " << (i + 1) << ") #######\n";
 		cerr << two->controller->inspectStateAsJson();
-		cerr << "\n";
+		cerr << "\n\n";
 		cerr.flush();
 	}
+}
+
+static void
+dumpControllerConfigsOnCrash(void *userData) {
+	WorkingObjects *wo = workingObjects;
+	unsigned int i;
 
 	for (i = 0; i < wo->threadWorkingObjects.size(); i++) {
 		ThreadWorkingObjects *two = &wo->threadWorkingObjects[i];
-		cerr << "### Request handler config (thread " << (i + 1) << ")\n";
+		cerr << "####### Controller config (thread " << (i + 1) << ") #######\n";
 		cerr << two->controller->inspectConfig();
-		cerr << "\n";
+		cerr << "\n\n";
 		cerr.flush();
 	}
+}
 
-	cerr << "### Pool state (simple)\n";
+static void
+dumpPoolStateOnCrash(void *userData) {
+	WorkingObjects *wo = workingObjects;
+
+	cerr << "####### Pool state (simple) #######\n";
 	// Do not lock, the crash may occur within the pool.
 	Pool::InspectOptions options(Pool::InspectOptions::makeAuthorized());
 	options.verbose = true;
 	cerr << wo->appPool->inspect(options, false);
-	cerr << "\n";
+	cerr << "\n\n";
 	cerr.flush();
 
-	cerr << "### mbuf stats\n\n";
+	cerr << "####### Pool state (XML) #######\n";
+	Pool::ToXmlOptions options2(Pool::ToXmlOptions::makeAuthorized());
+	options2.secrets = true;
+	cerr << wo->appPool->toXml(options2, false);
+	cerr << "\n\n";
+	cerr.flush();
+}
+
+static void
+dumpMbufStatsOnCrash(void *userData) {
+	WorkingObjects *wo = workingObjects;
 	cerr << "nfree_mbuf_blockq  : " <<
 		wo->threadWorkingObjects[0].serverKitContext->mbuf_pool.nfree_mbuf_blockq << "\n";
 	cerr << "nactive_mbuf_blockq: " <<
@@ -594,13 +626,6 @@ dumpDiagnosticsOnCrash(void *userData) {
 		wo->threadWorkingObjects[0].serverKitContext->mbuf_pool.mbuf_block_chunk_size << "\n";
 	cerr << "\n";
 	cerr.flush();
-
-	cerr << "### Pool state (XML)\n";
-	Pool::ToXmlOptions options2(Pool::ToXmlOptions::makeAuthorized());
-	options2.secrets = true;
-	cerr << wo->appPool->toXml(options2, false);
-	cerr << "\n\n";
-	cerr.flush();
 }
 
 static void
@@ -655,6 +680,7 @@ initializeNonPrivilegedWorkingObjects() {
 	wo->spawningKitContext = boost::make_shared<SpawningKit::Context>(
 		wo->spawningKitContextSchema);
 	wo->spawningKitContext->resourceLocator = &wo->resourceLocator;
+	wo->spawningKitContext->wrapperRegistry = coreWrapperRegistry;
 	wo->spawningKitContext->randomGenerator = wo->randomGenerator;
 	wo->spawningKitContext->integrationMode = coreConfig->get("integration_mode").asString();
 	wo->spawningKitContext->instanceDir = coreConfig->get("instance_dir").asString();
@@ -715,6 +741,7 @@ initializeNonPrivilegedWorkingObjects() {
 			&wo->singleAppModeConfig,
 			coreSchema->controllerSingleAppMode.translator);
 		two.controller->resourceLocator = &wo->resourceLocator;
+		two.controller->wrapperRegistry = coreWrapperRegistry;
 		two.controller->appPool = wo->appPool;
 		two.controller->shutdownFinishCallback = controllerShutdownFinished;
 		two.controller->initialize();
@@ -823,6 +850,27 @@ initializeSecurityUpdateChecker() {
 	checker->start();
 }
 
+static void
+initializeTelemetryCollector() {
+	return; // disable for now
+	TRACE_POINT();
+	WorkingObjects &wo = *workingObjects;
+
+	Json::Value config = coreConfig->inspectEffectiveValues();
+	TelemetryCollector *collector = new TelemetryCollector(
+		coreSchema->telemetryCollector.schema,
+		coreConfig->inspectEffectiveValues(),
+		coreSchema->telemetryCollector.translator);
+	wo.telemetryCollector = collector;
+	for (unsigned int i = 0; i < wo.threadWorkingObjects.size(); i++) {
+		ThreadWorkingObjects *two = &wo.threadWorkingObjects[i];
+		collector->controllers.push_back(two->controller);
+	}
+	collector->initialize();
+	collector->start();
+	wo.shutdownCounter.fetch_add(1, boost::memory_order_relaxed);
+}
+
 static void
 runAdminPanelConnector(AdminPanelConnector *connector) {
 	connector->run();
@@ -970,6 +1018,50 @@ reportInitializationInfo() {
 	}
 }
 
+static void
+initializeAbortHandlerCustomerDiagnostics() {
+	if (!Agent::Fundamentals::abortHandlerInstalled()) {
+		return;
+	}
+
+	Agent::Fundamentals::AbortHandlerConfig::DiagnosticsDumper *diagnosticsDumpers
+		= &Agent::Fundamentals::context->abortHandlerConfig.diagnosticsDumpers[0];
+
+	diagnosticsDumpers[0].name = "OXT backtraces";
+	diagnosticsDumpers[0].logFileName = "backtrace_oxt.log";
+	diagnosticsDumpers[0].func = dumpOxtBacktracesOnCrash;
+
+	diagnosticsDumpers[1].name = "controller states";
+	diagnosticsDumpers[1].logFileName = "controller_states.log";
+	diagnosticsDumpers[1].func = dumpControllerStatesOnCrash;
+
+	diagnosticsDumpers[2].name = "controller configs";
+	diagnosticsDumpers[2].logFileName = "controller_configs.log";
+	diagnosticsDumpers[2].func = dumpControllerConfigsOnCrash;
+
+	diagnosticsDumpers[3].name = "pool state";
+	diagnosticsDumpers[3].logFileName = "pool.log";
+	diagnosticsDumpers[3].func = dumpPoolStateOnCrash;
+
+	diagnosticsDumpers[4].name = "mbuf statistics";
+	diagnosticsDumpers[4].logFileName = "mbufs.log";
+	diagnosticsDumpers[4].func = dumpMbufStatsOnCrash;
+
+	Agent::Fundamentals::abortHandlerConfigChanged();
+}
+
+static void
+uninstallAbortHandlerCustomDiagnostics() {
+	if (!Agent::Fundamentals::abortHandlerInstalled()) {
+		return;
+	}
+
+	for (unsigned int i = 0; i < Agent::Fundamentals::AbortHandlerConfig::MAX_DIAGNOSTICS_DUMPERS; i++) {
+		Agent::Fundamentals::context->abortHandlerConfig.diagnosticsDumpers[i].func = NULL;
+	}
+	Agent::Fundamentals::abortHandlerConfigChanged();
+}
+
 static void
 mainLoop() {
 	TRACE_POINT();
@@ -980,9 +1072,6 @@ mainLoop() {
 			&& maxCpus <= CPU_SETSIZE;
 	#endif
 
-	Agent::Fundamentals::context->abortHandlerConfig.diagnosticsDumper = dumpDiagnosticsOnCrash;
-	Agent::Fundamentals::abortHandlerConfigChanged();
-
 	for (unsigned int i = 0; i < wo->threadWorkingObjects.size(); i++) {
 		ThreadWorkingObjects *two = &wo->threadWorkingObjects[i];
 		two->bgloop->start("Main event loop: thread " + toString(i + 1), 0);
@@ -1066,6 +1155,20 @@ apiServerShutdownFinished(Core::ApiServer::ApiServer *server) {
 	serverShutdownFinished();
 }
 
+static void
+telemetryCollectorAsyncShutdownThreadMain() {
+	WorkingObjects *wo = workingObjects;
+	wo->telemetryCollector->stop();
+	serverShutdownFinished();
+}
+
+static void
+asyncShutdownTelemetryCollector() {
+	oxt::thread(telemetryCollectorAsyncShutdownThreadMain,
+		"Telemetry collector shutdown",
+		512 * 1024);
+}
+
 /* Wait until the watchdog closes the feedback fd (meaning it
  * was killed) or until we receive an exit message.
  */
@@ -1087,8 +1190,7 @@ waitForExitEvent() {
 	TRACE_POINT();
 	if (syscalls::select(largestFd + 1, &fds, NULL, NULL, NULL) == -1) {
 		int e = errno;
-		Agent::Fundamentals::context->abortHandlerConfig.diagnosticsDumper = NULL;
-		Agent::Fundamentals::abortHandlerConfigChanged();
+		uninstallAbortHandlerCustomDiagnostics();
 		throw SystemException("select() failed", e);
 	}
 
@@ -1124,6 +1226,9 @@ waitForExitEvent() {
 		if (wo->apiWorkingObjects.apiServer != NULL) {
 			wo->apiWorkingObjects.bgloop->safe->runLater(shutdownApiServer);
 		}
+		if (wo->telemetryCollector != NULL) {
+			asyncShutdownTelemetryCollector();
+		}
 		if (wo->adminPanelConnector != NULL) {
 			wo->adminPanelConnector->asyncShutdown();
 		}
@@ -1135,8 +1240,7 @@ waitForExitEvent() {
 			&fds, NULL, NULL, NULL) == -1)
 		{
 			int e = errno;
-			Agent::Fundamentals::context->abortHandlerConfig.diagnosticsDumper = NULL;
-			Agent::Fundamentals::abortHandlerConfigChanged();
+			uninstallAbortHandlerCustomDiagnostics();
 			throw SystemException("select() failed", e);
 		}
 
@@ -1152,8 +1256,7 @@ cleanup() {
 	P_DEBUG("Shutting down " SHORT_PROGRAM_NAME " core...");
 	wo->appPool->destroy();
 
-	Agent::Fundamentals::context->abortHandlerConfig.diagnosticsDumper = dumpDiagnosticsOnCrash;
-	Agent::Fundamentals::abortHandlerConfigChanged();
+	uninstallAbortHandlerCustomDiagnostics();
 
 	for (unsigned i = 0; i < wo->threadWorkingObjects.size(); i++) {
 		ThreadWorkingObjects *two = &wo->threadWorkingObjects[i];
@@ -1162,6 +1265,11 @@ cleanup() {
 	if (wo->apiWorkingObjects.apiServer != NULL) {
 		wo->apiWorkingObjects.bgloop->stop();
 	}
+	if (wo->telemetryCollector != NULL
+	&& !coreConfig->get("telemetry_collector_disabled").asBool())
+	{
+		wo->telemetryCollector->runOneCycle(true);
+	}
 	wo->appPool.reset();
 	for (unsigned i = 0; i < wo->threadWorkingObjects.size(); i++) {
 		ThreadWorkingObjects *two = &wo->threadWorkingObjects[i];
@@ -1212,12 +1320,14 @@ runCore() {
 		initializeCurl();
 		initializeNonPrivilegedWorkingObjects();
 		initializeSecurityUpdateChecker();
+		initializeTelemetryCollector();
 		initializeAdminPanelConnector();
 		prestartWebApps();
 
 		UPDATE_TRACE_POINT();
 		warnIfPassengerRootVulnerable();
 		reportInitializationInfo();
+		initializeAbortHandlerCustomerDiagnostics();
 		mainLoop();
 
 		UPDATE_TRACE_POINT();
@@ -1279,7 +1389,9 @@ int
 coreMain(int argc, char *argv[]) {
 	int ret;
 
-	coreSchema = new Schema();
+	coreWrapperRegistry = new WrapperRegistry::Registry();
+	coreWrapperRegistry->finalize();
+	coreSchema = new Schema(coreWrapperRegistry);
 	coreConfig = new ConfigKit::Store(*coreSchema);
 	initializeAgent(argc, &argv, SHORT_PROGRAM_NAME " core",
 		*coreConfig, coreSchema->loggingKit.translator,
@@ -1291,5 +1403,6 @@ coreMain(int argc, char *argv[]) {
 
 	ret = runCore();
 	shutdownAgent(coreSchema, coreConfig);
+	delete coreWrapperRegistry;
 	return ret;
 }
