@@ -1,6 +1,6 @@
 /*
  *  Phusion Passenger - https://www.phusionpassenger.com/
- *  Copyright (c) 2011-2017 Phusion Holding B.V.
+ *  Copyright (c) 2011-2018 Phusion Holding B.V.
  *
  *  "Passenger", "Phusion Passenger" and "Union Station" are registered
  *  trademarks of Phusion Holding B.V.
@@ -29,14 +29,15 @@
 #include <oxt/tracable_exception.hpp>
 #include <string>
 #include <stdexcept>
+#include <limits>
 
 #include <Constants.h>
 #include <Exceptions.h>
 #include <LoggingKit/LoggingKit.h>
 #include <DataStructures/StringKeyTable.h>
 #include <ProcessManagement/Spawn.h>
-#include <Utils/StrIntUtils.h>
-#include <Utils/SystemMetricsCollector.h>
+#include <SystemTools/SystemMetricsCollector.h>
+#include <StrIntTools/StrIntUtils.h>
 #include <Core/SpawningKit/Config.h>
 #include <Core/SpawningKit/Journey.h>
 
@@ -602,7 +603,7 @@ private:
 					" scripts.</p>");
 				break;
 			case SUBPROCESS_APP_LOAD_OR_EXEC:
-				if (config.appType == "node") {
+				if (config.appType == "nodejs") {
 					message.append(
 						"<h3>Check whether the application calls <code>http.Server.listen()</code></h3>"
 						"<p>" SHORT_PROGRAM_NAME " requires that the application calls"
@@ -736,12 +737,13 @@ private:
 		const char *command[] = { "/bin/sh", "-c", "ulimit -a", NULL };
 		try {
 			SubprocessInfo info;
-			string result;
-			runCommandAndCaptureOutput(command, info, result);
-			if (result.empty()) {
-				result.assign("Error: command 'ulimit -a' failed");
+			SubprocessOutput output;
+			runCommandAndCaptureOutput(command, info, output,
+				std::numeric_limits<size_t>::max());
+			if (output.data.empty()) {
+				output.data.assign("Error: command 'ulimit -a' failed");
 			}
-			return result;
+			return output.data;
 		} catch (const SystemException &e) {
 			return P_STATIC_STRING("Error: command 'ulimit -a' failed: ") + e.what();
 		}
@@ -751,12 +753,13 @@ private:
 		const char *command[] = { "id", "-a", NULL };
 		try {
 			SubprocessInfo info;
-			string result;
-			runCommandAndCaptureOutput(command, info, result);
-			if (result.empty()) {
-				result.assign("Error: command 'id -a' failed");
+			SubprocessOutput output;
+			runCommandAndCaptureOutput(command, info, output,
+				std::numeric_limits<size_t>::max());
+			if (output.data.empty()) {
+				output.data.assign("Error: command 'id -a' failed");
 			}
-			return result;
+			return output.data;
 		} catch (const SystemException &e) {
 			return P_STATIC_STRING("Error: command 'id -a' failed: ") + e.what();
 		}
