@@ -52,8 +52,8 @@
 #include <FileTools/PathManip.h>
 #include <Utils.h>
 #include <Utils/ScopeGuard.h>
-#include <Utils/SystemTime.h>
-#include <Utils/StrIntUtils.h>
+#include <SystemTools/SystemTime.h>
+#include <StrIntTools/StrIntUtils.h>
 #include <Core/SpawningKit/Config.h>
 #include <Core/SpawningKit/Exceptions.h>
 #include <Core/SpawningKit/Handshake/BackgroundIOCapturer.h>
@@ -297,6 +297,7 @@ private:
 		result.stdoutAndErrFd = stdoutAndErrFd;
 		result.spawnEndTime = SystemTime::getUsec();
 		result.spawnEndTimeMonotonic = SystemTime::getMonotonicUsec();
+		setResultType(result);
 
 		if (socketIsNowPingable) {
 			assert(config->genericApp || config->findFreePort);
@@ -600,8 +601,8 @@ private:
 		if (s.st_uid != session.uid) {
 			errors.push_back("'sockets[" + toString(index)
 				+ "].address', when referring to a Unix domain socket file,"
-				" must be owned by user " + getUserName(session.uid)
-				+ " (actual owner: " + getUserName(s.st_uid) + ")");
+				" must be owned by user " + lookupSystemUsernameByUid(session.uid)
+				+ " (actual owner: " + lookupSystemUsernameByUid(s.st_uid) + ")");
 		}
 	}
 
@@ -1674,6 +1675,16 @@ private:
 		return result;
 	}
 
+	void setResultType(Result &result) const {
+		if (config->genericApp) {
+			result.type = Result::GENERIC;
+		} else if (config->startsUsingWrapper) {
+			result.type = Result::AUTO_SUPPORTED;
+		} else {
+			result.type = Result::KURIA;
+		}
+	}
+
 public:
 	struct DebugSupport {
 		virtual ~DebugSupport() { }
