@@ -1,6 +1,6 @@
 /*
  *  Phusion Passenger - https://www.phusionpassenger.com/
- *  Copyright (c) 2014-2015 Phusion Holding B.V.
+ *  Copyright (c) 2014-2018 Phusion Holding B.V.
  *
  *  "Passenger", "Phusion Passenger" and "Union Station" are registered
  *  trademarks of Phusion Holding B.V.
@@ -29,6 +29,8 @@
 #include <psg_sysqueue.h>
 
 #include <boost/cstdint.hpp>
+#include <boost/config.hpp>
+#include <boost/scoped_ptr.hpp>
 #include <oxt/system_calls.hpp>
 #include <oxt/backtrace.hpp>
 #include <oxt/macros.hpp>
@@ -36,6 +38,12 @@
 #include <new>
 #include <ev++.h>
 
+// for std::swap()
+#if __cplusplus >= 201103L
+	#include <utility>
+#else
+	#include <algorithm>
+#endif
 #include <sys/socket.h>
 #include <sys/un.h>
 #include <netinet/in.h>
@@ -48,7 +56,7 @@
 #include <jsoncpp/json.h>
 #include <SmallVector.h>
 
-#include <Logging.h>
+#include <LoggingKit/LoggingKit.h>
 #include <SafeLibev.h>
 #include <Constants.h>
 #include <ServerKit/Context.h>
@@ -56,10 +64,13 @@
 #include <ServerKit/Hooks.h>
 #include <ServerKit/Client.h>
 #include <ServerKit/ClientRef.h>
+#include <ConfigKit/ConfigKit.h>
+#include <Algorithms/MovingAverage.h>
 #include <Utils.h>
 #include <Utils/ScopeGuard.h>
-#include <Utils/StrIntUtils.h>
-#include <Utils/IOUtils.h>
+#include <StrIntTools/StrIntUtils.h>
+#include <IOTools/IOUtils.h>
+#include <SystemTools/SystemTime.h>
 
 namespace Passenger {
 namespace ServerKit {
@@ -70,7 +81,7 @@ using namespace oxt;
 
 
 // We use 'this' so that the macros work in derived template classes like HttpServer<>.
-#define SKS_LOG(level, file, line, expr)  P_LOG(level, file, line, "[" << this->getServerName() << "] " << expr)
+#define SKS_LOG(level, file, line, expr)  P_LOG(Passenger::LoggingKit::context, level, file, line, "[" << this->getServerName() << "] " << expr)
 #define SKS_ERROR(expr)  P_ERROR("[" << this->getServerName() << "] " << expr)
 #define SKS_WARN(expr)   P_WARN("[" << this->getServerName() << "] " << expr)
 #define SKS_INFO(expr)   P_INFO("[" << this->getServerName() << "] " << expr)
@@ -94,26 +105,26 @@ using namespace oxt;
 
 #define SKC_LOG_FROM_STATIC(server, client, level, expr) \
 	do { \
-		if (Passenger::getLogLevel() >= level) { \
+		if (Passenger::LoggingKit::getLevel() >= level) { \
 			char _clientName[16]; \
 			int _clientNameSize = server->getClientName((client), _clientName, sizeof(_clientName)); \
-			P_LOG(level, __FILE__, __LINE__, \
+			P_LOG(LoggingKit::context, level, __FILE__, __LINE__, \
 				"[Client " << StaticString(_clientName, _clientNameSize) << "] " << expr); \
 		} \
 	} while (0)
 #define SKC_ERROR_FROM_STATIC(server, client, expr) \
-	SKC_LOG_FROM_STATIC(server, client, LVL_ERROR, expr)
+	SKC_LOG_FROM_STATIC(server, client, Passenger::LoggingKit::ERROR, expr)
 #define SKC_WARN_FROM_STATIC(server, client, expr) \
-	SKC_LOG_FROM_STATIC(server, client, LVL_WARN, expr)
+	SKC_LOG_FROM_STATIC(server, client, Passenger::LoggingKit::WARN, expr)
 #define SKC_NOTICE_FROM_STATIC(server, client, expr) \
-	SKC_LOG_FROM_STATIC(server, client, LVL_NOTICE, expr)
+	SKC_LOG_FROM_STATIC(server, client, Passenger::LoggingKit::NOTICE, expr)
 #define SKC_INFO_FROM_STATIC(server, client, expr) \
-	SKC_LOG_FROM_STATIC(server, client, LVL_INFO, expr)
+	SKC_LOG_FROM_STATIC(server, client, Passenger::LoggingKit::INFO, expr)
 #define SKC_DEBUG_FROM_STATIC(server, client, expr) \
-	SKC_LOG_FROM_STATIC(server, client, LVL_DEBUG, expr)
+	SKC_LOG_FROM_STATIC(server, client, Passenger::LoggingKit::DEBUG, expr)
 #define SKC_DEBUG_FROM_STATIC_WITH_POS(server, client, file, line, expr) \
 	do { \
-		if (OXT_UNLIKELY(Passenger::getLogLevel() >= LVL_DEBUG)) { \
+		if (OXT_UNLIKELY(Passenger::LoggingKit::getLevel() >= Passenger::LoggingKit::DEBUG)) { \
 			char _clientName[16]; \
 			int _clientNameSize = server->getClientName((client), _clientName, sizeof(_clientName)); \
 			P_DEBUG_WITH_POS(file, line, \
@@ -124,7 +135,7 @@ using namespace oxt;
 	SKC_TRACE_FROM_STATIC_WITH_POS(server, client, level, __FILE__, __LINE__, expr)
 #define SKC_TRACE_FROM_STATIC_WITH_POS(server, client, level, file, line, expr) \
 	do { \
-		if (OXT_UNLIKELY(Passenger::getLogLevel() >= level)) { \
+		if (OXT_UNLIKELY(Passenger::LoggingKit::getLevel() >= Passenger::LoggingKit::INFO + level)) { \
 			char _clientName[16]; \
 			int _clientNameSize = server->getClientName((client), _clientName, sizeof(_clientName)); \
 			P_TRACE_WITH_POS(level, file, line, \
@@ -137,6 +148,77 @@ using namespace oxt;
 	TRACE_POINT_WITH_DATA_FUNCTION(klass::_getClientNameFromTracePoint, client); \
 	SKC_TRACE_FROM_STATIC(server, client, 3, "Event: " eventName)
 
+
+/*
+ * BEGIN ConfigKit schema: Passenger::ServerKit::BaseServerSchema
+ * (do not edit: following text is automatically generated
+ * by 'rake configkit_schemas_inline_comments')
+ *
+ *   accept_burst_count           unsigned integer   -   default(32)
+ *   client_freelist_limit        unsigned integer   -   default(0)
+ *   min_spare_clients            unsigned integer   -   default(0)
+ *   start_reading_after_accept   boolean            -   default(true)
+ *
+ * END
+ */
+class BaseServerSchema: public ConfigKit::Schema {
+private:
+	void initialize() {
+		using namespace ConfigKit;
+
+		add("accept_burst_count", UINT_TYPE, OPTIONAL, 32);
+		add("start_reading_after_accept", BOOL_TYPE, OPTIONAL, true);
+		add("min_spare_clients", UINT_TYPE, OPTIONAL, 0);
+		add("client_freelist_limit", UINT_TYPE, OPTIONAL, 0);
+	}
+
+public:
+	BaseServerSchema() {
+		initialize();
+		finalize();
+	}
+
+	BaseServerSchema(bool _subclassing) {
+		initialize();
+	}
+};
+
+struct BaseServerConfigRealization {
+	unsigned int acceptBurstCount: 7;
+	bool startReadingAfterAccept: 1;
+	unsigned int minSpareClients: 12;
+	unsigned int clientFreelistLimit: 12;
+
+	BaseServerConfigRealization(const ConfigKit::Store &config)
+		: acceptBurstCount(config["accept_burst_count"].asUInt()),
+		  startReadingAfterAccept(config["start_reading_after_accept"].asBool()),
+		  minSpareClients(config["min_spare_clients"].asUInt()),
+		  clientFreelistLimit(config["client_freelist_limit"].asUInt())
+		{ }
+
+	void swap(BaseServerConfigRealization &other) BOOST_NOEXCEPT_OR_NOTHROW {
+		#define SWAP_BITFIELD(Type, name) \
+			do { \
+				Type tmp = name; \
+				name = other.name; \
+				other.name = tmp; \
+			} while (false)
+
+		SWAP_BITFIELD(unsigned int, acceptBurstCount);
+		SWAP_BITFIELD(bool, startReadingAfterAccept);
+		SWAP_BITFIELD(unsigned int, minSpareClients);
+		SWAP_BITFIELD(unsigned int, clientFreelistLimit);
+
+		#undef SWAP_BITFIELD
+	}
+};
+
+struct BaseServerConfigChangeRequest {
+	boost::scoped_ptr<ConfigKit::Store> config;
+	boost::scoped_ptr<BaseServerConfigRealization> configRlz;
+};
+
+
 /**
  * A highly optimized generic base class for evented socket servers, implementing basic,
  * low-level connection management.
@@ -199,12 +281,11 @@ public:
 	static const unsigned int MAX_ACCEPT_BURST_COUNT = 127;
 
 	typedef void (*Callback)(DerivedServer *server);
+	typedef BaseServerConfigChangeRequest ConfigChangeRequest;
 
 	/***** Configuration *****/
-	unsigned int acceptBurstCount: 7;
-	bool startReadingAfterAccept: 1;
-	unsigned int minSpareClients: 12;
-	unsigned int clientFreelistLimit: 12;
+	ConfigKit::Store config;
+	BaseServerConfigRealization configRlz;
 	Callback shutdownFinishCallback;
 
 	/***** Working state and statistics (do not modify) *****/
@@ -212,8 +293,11 @@ public:
 	FreeClientList freeClients;
 	ClientList activeClients, disconnectedClients;
 	unsigned int freeClientCount, activeClientCount, disconnectedClientCount;
-	unsigned long totalClientsAccepted;
+	unsigned int peakActiveClientCount;
+	unsigned long totalClientsAccepted, lastTotalClientsAccepted;
 	unsigned long long totalBytesConsumed;
+	ev_tstamp lastStatisticsUpdateTime;
+	double clientAcceptSpeed1m, clientAcceptSpeed1h;
 
 private:
 	Context *ctx;
@@ -221,11 +305,28 @@ private:
 	uint8_t nEndpoints: 3;
 	bool accept4Available: 1;
 	ev::timer acceptResumptionWatcher;
+	ev::timer statisticsUpdateWatcher;
 	ev::io endpoints[SERVER_KIT_MAX_SERVER_ENDPOINTS];
 
 
 	/***** Private methods *****/
 
+	void preinitialize(Context *context) {
+		STAILQ_INIT(&freeClients);
+		TAILQ_INIT(&activeClients);
+		TAILQ_INIT(&disconnectedClients);
+
+		acceptResumptionWatcher.set(context->libev->getLoop());
+		acceptResumptionWatcher.set<
+			BaseServer<DerivedServer, Client>,
+			&BaseServer<DerivedServer, Client>::onAcceptResumeTimeout>(this);
+
+		statisticsUpdateWatcher.set(context->libev->getLoop());
+		statisticsUpdateWatcher.set<
+			BaseServer<DerivedServer, Client>,
+			&BaseServer<DerivedServer, Client>::onStatisticsUpdateTimeout>(this);
+	}
+
 	static void _onAcceptable(EV_P_ ev_io *io, int revents) {
 		static_cast<BaseServer *>(io->data)->onAcceptable(io, revents);
 	}
@@ -241,7 +342,7 @@ private:
 		P_ASSERT_EQ(serverState, ACTIVE);
 		SKS_DEBUG("New clients can be accepted on a server socket");
 
-		for (unsigned int i = 0; i < acceptBurstCount; i++) {
+		for (unsigned int i = 0; i < configRlz.acceptBurstCount; i++) {
 			fd = acceptNonBlockingSocket(io->fd);
 			if (fd == -1) {
 				error = true;
@@ -335,6 +436,16 @@ private:
 		}
 	}
 
+	void onStatisticsUpdateTimeout(ev::timer &timer, int revents) {
+		TRACE_POINT();
+
+		this->onUpdateStatistics();
+		this->onFinalizeStatisticsUpdate();
+
+		timer.repeat = timeToNextMultipleD(5, ev_now(this->getLoop()));
+		timer.again();
+	}
+
 	unsigned int getNextClientNumber() {
 		return nextClientNumber++;
 	}
@@ -399,7 +510,7 @@ private:
 	}
 
 	bool addClientToFreelist(Client *client) {
-		if (freeClientCount < clientFreelistLimit) {
+		if (freeClientCount < configRlz.clientFreelistLimit) {
 			STAILQ_INSERT_HEAD(&freeClients, client, nextClient.freeClient);
 			freeClientCount++;
 			client->refcount.store(2, boost::memory_order_relaxed);
@@ -440,11 +551,14 @@ private:
 
 	void finishShutdown() {
 		TRACE_POINT();
-		compact(LVL_INFO);
+		compact(LoggingKit::INFO);
+
+		acceptResumptionWatcher.stop();
+		statisticsUpdateWatcher.stop();
 
 		SKS_NOTICE("Shutdown finished");
 		serverState = FINISHED_SHUTDOWN;
-		if (shutdownFinishCallback != NULL) {
+		if (shutdownFinishCallback) {
 			shutdownFinishCallback(static_cast<DerivedServer *>(this));
 		}
 	}
@@ -513,12 +627,14 @@ protected:
 	virtual void onClientsAccepted(Client **clients, unsigned int size) {
 		unsigned int i;
 
+		peakActiveClientCount = std::max(peakActiveClientCount, activeClientCount);
+
 		for (i = 0; i < size; i++) {
 			Client *client = clients[i];
 
 			onClientAccepted(client);
 			if (client->connected()) {
-				if (startReadingAfterAccept) {
+				if (configRlz.startReadingAfterAccept) {
 					client->input.startReading();
 				} else {
 					client->input.startReadingInNextTick();
@@ -566,10 +682,33 @@ protected:
 			getClientOutputErrorDisconnectionLogLevel(client, errcode));
 	}
 
-	virtual PassengerLogLevel getClientOutputErrorDisconnectionLogLevel(
+	virtual LoggingKit::Level getClientOutputErrorDisconnectionLogLevel(
 		Client *client, int errcode) const
 	{
-		return LVL_WARN;
+		return LoggingKit::WARN;
+	}
+
+	virtual void onUpdateStatistics() {
+		SKS_DEBUG("Updating statistics");
+		ev_tstamp now = ev_now(this->getLoop());
+		ev_tstamp duration = now - lastStatisticsUpdateTime;
+
+		// Statistics are updated about every 5 seconds, so about 12 updates
+		// per minute. We want the old average to decay to 5% after 1 minute
+		// and 1 hour, respectively, so:
+		// 1 minute: 1 - exp(ln(0.05) / 12) = 0.22092219194555585
+		// 1 hour  : 1 - exp(ln(0.05) / (60 * 12)) = 0.0041520953856636345
+		clientAcceptSpeed1m = expMovingAverage(clientAcceptSpeed1m,
+			(totalClientsAccepted - lastTotalClientsAccepted) / duration,
+			0.22092219194555585);
+		clientAcceptSpeed1h = expMovingAverage(clientAcceptSpeed1h,
+			(totalClientsAccepted - lastTotalClientsAccepted) / duration,
+			0.0041520953856636345);
+	}
+
+	virtual void onFinalizeStatisticsUpdate() {
+		lastTotalClientsAccepted = totalClientsAccepted;
+		lastStatisticsUpdateTime = ev_now(this->getLoop());
 	}
 
 	virtual void reinitializeClient(Client *client, int fd) {
@@ -591,30 +730,29 @@ protected:
 public:
 	/***** Public methods *****/
 
-	BaseServer(Context *context)
-		: acceptBurstCount(32),
-		  startReadingAfterAccept(true),
-		  minSpareClients(0),
-		  clientFreelistLimit(0),
+	BaseServer(Context *context, const BaseServerSchema &schema,
+		const Json::Value &initialConfig = Json::Value(),
+		const ConfigKit::Translator &translator = ConfigKit::DummyTranslator())
+		: config(schema, initialConfig, translator),
+		  configRlz(config),
 		  shutdownFinishCallback(NULL),
 		  serverState(ACTIVE),
 		  freeClientCount(0),
 		  activeClientCount(0),
 		  disconnectedClientCount(0),
+		  peakActiveClientCount(0),
 		  totalClientsAccepted(0),
+		  lastTotalClientsAccepted(0),
 		  totalBytesConsumed(0),
+		  lastStatisticsUpdateTime(ev_time()),
+		  clientAcceptSpeed1m(-1),
+		  clientAcceptSpeed1h(-1),
 		  ctx(context),
 		  nextClientNumber(1),
 		  nEndpoints(0),
 		  accept4Available(true)
 	{
-		STAILQ_INIT(&freeClients);
-		TAILQ_INIT(&activeClients);
-		TAILQ_INIT(&disconnectedClients);
-		acceptResumptionWatcher.set(context->libev->getLoop());
-		acceptResumptionWatcher.set<
-			BaseServer<DerivedServer, Client>,
-			&BaseServer<DerivedServer, Client>::onAcceptResumeTimeout>(this);
+		preinitialize(context);
 	}
 
 	virtual ~BaseServer() {
@@ -624,10 +762,15 @@ public:
 
 	/***** Initialization, listening and shutdown *****/
 
+	virtual void initialize() {
+		statisticsUpdateWatcher.set(5, 5);
+		statisticsUpdateWatcher.start();
+	}
+
 	// Pre-create multiple client objects so that they get allocated
 	// near each other in memory. Hopefully increases CPU cache locality.
 	void createSpareClients() {
-		for (unsigned int i = 0; i < minSpareClients; i++) {
+		for (unsigned int i = 0; i < configRlz.minSpareClients; i++) {
 			Client *client = createNewClientObject();
 			client->setConnState(Client::IN_FREELIST);
 			STAILQ_INSERT_HEAD(&freeClients, client, nextClient.freeClient);
@@ -743,7 +886,7 @@ public:
 
 	/***** Server management *****/
 
-	virtual void compact(int logLevel = LVL_NOTICE) {
+	virtual void compact(LoggingKit::Level logLevel = LoggingKit::NOTICE) {
 		unsigned int count = freeClientCount;
 
 		while (!STAILQ_EMPTY(&freeClients)) {
@@ -932,7 +1075,7 @@ public:
 	}
 
 	void disconnectWithError(Client **client, const StaticString &message,
-		PassengerLogLevel logLevel = LVL_WARN)
+		LoggingKit::Level logLevel = LoggingKit::WARN)
 	{
 		SKC_LOG(*client, logLevel, "Disconnecting client with error: " << message);
 		disconnect(client);
@@ -960,28 +1103,25 @@ public:
 		return P_STATIC_STRING("Server");
 	}
 
-	virtual void configure(const Json::Value &doc) {
-		if (doc.isMember("accept_burst_count")) {
-			acceptBurstCount = doc["accept_burst_count"].asUInt();
-		}
-		if (doc.isMember("start_reading_after_accept")) {
-			startReadingAfterAccept = doc["start_reading_after_accept"].asBool();
-		}
-		if (doc.isMember("min_spare_clients")) {
-			minSpareClients = doc["min_spare_clients"].asUInt();
-		}
-		if (doc.isMember("client_freelist_limit")) {
-			clientFreelistLimit = doc["client_freelist_limit"].asUInt();
+	bool prepareConfigChange(const Json::Value &updates,
+		vector<ConfigKit::Error> &errors, BaseServerConfigChangeRequest &req)
+	{
+		req.config.reset(new ConfigKit::Store(config, updates, errors));
+		if (errors.empty()) {
+			req.configRlz.reset(new BaseServerConfigRealization(*req.config));
 		}
+		return errors.empty();
 	}
 
-	virtual Json::Value getConfigAsJson() const {
-		Json::Value doc;
-		doc["accept_burst_count"] = acceptBurstCount;
-		doc["start_reading_after_accept"] = startReadingAfterAccept;
-		doc["min_spare_clients"] = minSpareClients;
-		doc["client_freelist_limit"] = clientFreelistLimit;
-		return doc;
+	void commitConfigChange(BaseServerConfigChangeRequest &req)
+		BOOST_NOEXCEPT_OR_NOTHROW
+	{
+		config.swap(*req.config);
+		configRlz.swap(*req.configRlz);
+	}
+
+	virtual Json::Value inspectConfig() const {
+		return config.inspect();
 	}
 
 	virtual Json::Value inspectStateAsJson() const {
@@ -995,6 +1135,13 @@ public:
 		doc["active_client_count"] = activeClientCount;
 		Json::Value &disconnectedClientsDoc = doc["disconnected_clients"] = Json::Value(Json::objectValue);
 		doc["disconnected_client_count"] = disconnectedClientCount;
+		doc["peak_active_client_count"] = peakActiveClientCount;
+		doc["client_accept_speed"]["1m"] = averageSpeedToJson(
+			capFloatPrecision(clientAcceptSpeed1m * 60),
+			"minute", "1 minute", -1);
+		doc["client_accept_speed"]["1h"] = averageSpeedToJson(
+			capFloatPrecision(clientAcceptSpeed1h * 60),
+			"minute", "1 hour", -1);
 		doc["total_clients_accepted"] = (Json::UInt64) totalClientsAccepted;
 		doc["total_bytes_consumed"] = (Json::UInt64) totalBytesConsumed;
 
@@ -1097,8 +1244,9 @@ public:
 template <typename Client = Client>
 class Server: public BaseServer<Server<Client>, Client> {
 public:
-	Server(Context *context)
-		: BaseServer<Server, Client>(context)
+	Server(Context *context, const BaseServerSchema &schema,
+		const Json::Value &initialConfig = Json::Value())
+		: BaseServer<Server, Client>(context, schema, initialConfig)
 		{ }
 };
 
